print("import: 'conda_smithy'")
import conda_smithy

print("import: 'conda_smithy.ci_register'")
import conda_smithy.ci_register

print("import: 'conda_smithy.configure_feedstock'")
import conda_smithy.configure_feedstock




# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import datetime
import io
import itertools
import os
import re
import time

import github
import jinja2
import ruamel.yaml

from conda_build.metadata import (ensure_valid_license_family,
                                  FIELDS as cbfields)
import conda_build.conda_interface

from collections import defaultdict

import copy

FIELDS = copy.deepcopy(cbfields)

# Just in case 'extra' moves into conda_build
if 'extra' not in FIELDS.keys():
    FIELDS['extra'] = []

FIELDS['extra'].append('recipe-maintainers')

EXPECTED_SECTION_ORDER = ['package', 'source', 'build', 'requirements',
                          'test', 'app', 'outputs', 'about', 'extra']

REQUIREMENTS_ORDER = ['build', 'host', 'run']

TEST_KEYS = {'imports', 'commands'}

sel_pat = re.compile(r'(.+?)\s*(#.*)?\[([^\[\]]+)\](?(2).*)$')
jinja_pat = re.compile(r'\s*\{%\s*(set)\s+[^\s]+\s*=\s*[^\s]+\s*%\}')


class NullUndefined(jinja2.Undefined):
    def __unicode__(self):
        return self._undefined_name

    def __getattr__(self, name):
        return '{}.{}'.format(self, name)

    def __getitem__(self, name):
        return '{}["{}"]'.format(self, name)


def get_section(parent, name, lints):
    section = parent.get(name, {})
    if not isinstance(section, dict):
        lints.append('The "{}" section was expected to be a dictionary, but '
                     'got a {}.'.format(name, type(section).__name__))
        section = {}
    return section


def lint_section_order(major_sections, lints):
    section_order_sorted = sorted(major_sections,
                                  key=EXPECTED_SECTION_ORDER.index)
    if major_sections != section_order_sorted:
        section_order_sorted_str = map(lambda s: "'%s'" % s,
                                       section_order_sorted)
        section_order_sorted_str = ", ".join(section_order_sorted_str)
        section_order_sorted_str = "[" + section_order_sorted_str + "]"
        lints.append('The top level meta keys are in an unexpected order. '
                     'Expecting {}.'.format(section_order_sorted_str))


def lint_about_contents(about_section, lints):
    for about_item in ['home', 'license', 'summary']:
        # if the section doesn't exist, or is just empty, lint it.
        if not about_section.get(about_item, ''):
            lints.append('The {} item is expected in the about section.'
                         ''.format(about_item))


def lintify(meta, recipe_dir=None, conda_forge=False):
    lints = []
    major_sections = list(meta.keys())

    # If the recipe_dir exists (no guarantee within this function) , we can
    # find the meta.yaml within it.
    meta_fname = os.path.join(recipe_dir or '', 'meta.yaml')

    source_section = get_section(meta, 'source', lints)
    build_section = get_section(meta, 'build', lints)
    requirements_section = get_section(meta, 'requirements', lints)
    test_section = get_section(meta, 'test', lints)
    about_section = get_section(meta, 'about', lints)
    extra_section = get_section(meta, 'extra', lints)
    package_section = get_section(meta, 'package', lints)

    # 0: Top level keys should be expected
    unexpected_sections = []
    for section in major_sections:
        if section not in EXPECTED_SECTION_ORDER:
            lints.append('The top level meta key {} is unexpected' .format(section))
            unexpected_sections.append(section)

    for section in unexpected_sections:
        major_sections.remove(section)

    # 1: Top level meta.yaml keys should have a specific order.
    lint_section_order(major_sections, lints)

    # 2: The about section should have a home, license and summary.
    lint_about_contents(about_section, lints)

    # 3a: The recipe should have some maintainers.
    if not extra_section.get('recipe-maintainers', []):
        lints.append('The recipe could do with some maintainers listed in '
                     'the `extra/recipe-maintainers` section.')

    # 3b: Maintainers should be a list
    if not isinstance(extra_section.get('recipe-maintainers', []), list):
        lints.append('Recipe maintainers should be a json list.')

    # 4: The recipe should have some tests.
    if not any(key in TEST_KEYS for key in test_section):
        test_files = ['run_test.py', 'run_test.sh', 'run_test.bat',
                      'run_test.pl']
        a_test_file_exists = (recipe_dir is not None and
                              any(os.path.exists(os.path.join(recipe_dir,
                                                              test_file))
                                  for test_file in test_files))
        if not a_test_file_exists:
            lints.append('The recipe must have some tests.')

    # 5: License cannot be 'unknown.'
    license = about_section.get('license', '').lower()
    if 'unknown' == license.strip():
        lints.append('The recipe license cannot be unknown.')

    # 6: Selectors should be in a tidy form.
    if recipe_dir is not None and os.path.exists(meta_fname):
        bad_selectors = []
        bad_lines = []
        # Good selectors look like ".*\s\s#\s[...]"
        good_selectors_pat = re.compile(r'(.+?)\s{2,}#\s\[(.+)\](?(2).*)$')
        with io.open(meta_fname, 'rt') as fh:
            for selector_line, line_number in selector_lines(fh):
                if not good_selectors_pat.match(selector_line):
                    bad_selectors.append(selector_line)
                    bad_lines.append(line_number)
        if bad_selectors:
            lints.append('Selectors are suggested to take a '
                         '``<two spaces>#<one space>[<expression>]`` form.'
                         ' See lines {}'.format(bad_lines))

    # 7: The build section should have a build number.
    if build_section.get('number', None) is None:
        lints.append('The recipe must have a `build/number` section.')

    # 8: The build section should be before the run section in requirements.
    seen_requirements = [
            k for k in requirements_section if k in REQUIREMENTS_ORDER]
    requirements_order_sorted = sorted(seen_requirements,
                                       key=REQUIREMENTS_ORDER.index)
    if seen_requirements != requirements_order_sorted:
        lints.append('The `requirements/build` section should be defined '
                     'before the `requirements/run` section.')

    # 9: Files downloaded should have a hash.
    if ('url' in source_section and
            not ({'sha1', 'sha256', 'md5'} & set(source_section.keys()))):
        lints.append('When defining a source/url please add a sha256, sha1 '
                     'or md5 checksum (sha256 preferably).')

    # 10: License should not include the word 'license'.
    license = about_section.get('license', '').lower()
    if 'license' in license.lower():
        lints.append('The recipe `license` should not include the word '
                     '"License".')

    # 11: There should be one empty line at the end of the file.
    if recipe_dir is not None and os.path.exists(meta_fname):
        with io.open(meta_fname, 'r') as f:
            lines = f.read().split('\n')
        # Count the number of empty lines from the end of the file
        empty_lines = itertools.takewhile(lambda x: x == '', reversed(lines))
        end_empty_lines_count = len(list(empty_lines))
        if end_empty_lines_count > 1:
            lints.append('There are {} too many lines.  '
                         'There should be one empty line at the end of the '
                         'file.'.format(end_empty_lines_count - 1))
        elif end_empty_lines_count < 1:
            lints.append('There are too few lines.  There should be one empty '
                         'line at the end of the file.')

    # 12: License family must be valid (conda-build checks for that)
    try:
        ensure_valid_license_family(meta)
    except RuntimeError as e:
        lints.append(str(e))

    # 13: Check that the recipe name is valid
    recipe_name = package_section.get('name', '').strip()
    if re.match('^[a-z0-9_\-.]+$', recipe_name) is None:
        lints.append('Recipe name has invalid characters. only lowercase alpha, numeric, '
                     'underscores, hyphens and dots allowed')

    # 14: Run conda-forge specific lints
    if conda_forge:
        run_conda_forge_lints(meta, recipe_dir, lints)

    # 15: Check if we are using legacy patterns
    build_reqs = requirements_section.get('build', None)
    if build_reqs and ('numpy x.x' in build_reqs):
        lints.append('Using pinned numpy packages is a deprecated pattern.  Consider '
                     'using the method outlined '
                     '[here](https://conda-forge.org/docs/meta.html#building-against-numpy).')

    # 16: Subheaders should be in the allowed subheadings
    for section in major_sections:
        expected_subsections = FIELDS.get(section, [])
        if not expected_subsections:
            continue
        for subsection in get_section(meta, section, lints):
            if subsection not in expected_subsections:
                lints.append('The {} section contained an unexpected '
                             'subsection name. {} is not a valid subsection'
                             ' name.'.format(section, subsection))

    # 17: noarch doesn't work with selectors
    if build_section.get('noarch') is not None and os.path.exists(meta_fname):
        with io.open(meta_fname, 'rt') as fh:
            in_requirements = False
            for line in fh:
                line_s = line.strip()
                if (line_s == "requirements:"):
                    in_requirements = True
                    requirements_spacing = line[:-len(line.lstrip())]
                    continue
                if line_s.startswith("skip:") and is_selector_line(line):
                    lints.append("`noarch` packages can't have selectors. If "
                                 "the selectors are necessary, please remove "
                                 "`noarch: {}`.".format(build_section['noarch']))
                    break
                if in_requirements:
                    if requirements_spacing == line[:-len(line.lstrip())]:
                        in_requirements = False
                        continue
                    if is_selector_line(line):
                        lints.append("`noarch` packages can't have selectors. If "
                                     "the selectors are necessary, please remove "
                                     "`noarch: {}`.".format(build_section['noarch']))
                        break

    # 18: noarch and python setup.py doesn't work
    if build_section.get('noarch') == 'python':
        if 'script' in build_section:
            scripts = build_section['script']
            if isinstance(scripts, str):
                scripts = [scripts]
            for script in scripts:
                if "python setup.py install" in script:
                    lints.append("`noarch: python` packages should use pip. "
                                 "See https://conda-forge.org/docs/meta.html#use-pip")

    # 19: check version
    if package_section.get('version') is not None:
        ver = str(package_section.get('version'))
        try:
            conda_build.conda_interface.VersionOrder(ver)
        except:
            lints.append("Package version {} doesn't match conda spec".format(ver))

    # 20: Jinja2 variable definitions should be nice.
    if recipe_dir is not None and os.path.exists(meta_fname):
        bad_jinja = []
        bad_lines = []
        # Good Jinja2 variable definitions look like "{% set .+ = .+ %}"
        good_jinja_pat = re.compile(r'\s*\{%\s(set)\s[^\s]+\s=\s[^\s]+\s%\}')
        with io.open(meta_fname, 'rt') as fh:
            for jinja_line, line_number in jinja_lines(fh):
                if not good_jinja_pat.match(jinja_line):
                    bad_jinja.append(jinja_line)
                    bad_lines.append(line_number)
        if bad_jinja:
            lints.append('Jinja2 variable definitions are suggested to '
                         'take a ``{{%<one space>set<one space>'
                         '<variable name><one space>=<one space>'
                         '<expression><one space>%}}`` form. See lines '
                         '{}'.format(bad_lines))
    return lints


def run_conda_forge_lints(meta, recipe_dir, lints):
    gh = github.Github(os.environ['GH_TOKEN'])
    package_section = get_section(meta, 'package', lints)
    extra_section = get_section(meta, 'extra', lints)
    recipe_dirname = os.path.basename(recipe_dir) if recipe_dir else 'recipe'
    recipe_name = package_section.get('name', '').strip()
    is_staged_recipes = recipe_dirname != 'recipe'

    # 1: Check that the recipe does not exist in conda-forge
    if is_staged_recipes:
        cf = gh.get_user(os.getenv('GH_ORG', 'conda-forge'))
        try:
            cf.get_repo('{}-feedstock'.format(recipe_name))
            feedstock_exists = True
        except github.UnknownObjectException as e:
            feedstock_exists = False

        if feedstock_exists:
            lints.append('Feedstock with the same name exists in conda-forge')

        bio = gh.get_user('bioconda').get_repo('bioconda-recipes')
        try:
            bio.get_dir_contents('recipes/{}'.format(recipe_name))
        except github.UnknownObjectException as e:
            pass
        else:
            lints.append("Recipe with the same name exists in bioconda: "
                         "please discuss with @conda-forge/bioconda-recipes.")

    # 2: Check that the recipe maintainers exists:
    maintainers = extra_section.get('recipe-maintainers', [])
    for maintainer in maintainers:
        try:
            gh.get_user(maintainer)
        except github.UnknownObjectException as e:
            lints.append('Recipe maintainer "{}" does not exist'.format(maintainer))

    # 3: if the recipe dir is inside the example dir
    if recipe_dir is not None and 'recipes/example/' in recipe_dir:
        lints.append('Please move the recipe out of the example dir and '
                     'into its own dir.')


def is_selector_line(line):
    # Using the same pattern defined in conda-build (metadata.py),
    # we identify selectors.
    line = line.rstrip()
    if line.lstrip().startswith('#'):
        # Don't bother with comment only lines
        return False
    m = sel_pat.match(line)
    if m:
        m.group(3)
        return True
    return False


def is_jinja_line(line):
    line = line.rstrip()
    m = jinja_pat.match(line)
    if m:
        return True
    return False


def selector_lines(lines):
    for i, line in enumerate(lines):
        if is_selector_line(line):
            yield line, i


def jinja_lines(lines):
    for i, line in enumerate(lines):
        if is_jinja_line(line):
            yield line, i


def main(recipe_dir, conda_forge=False):
    recipe_dir = os.path.abspath(recipe_dir)
    recipe_meta = os.path.join(recipe_dir, 'meta.yaml')
    if not os.path.exists(recipe_dir):
        raise IOError('Feedstock has no recipe/meta.yaml.')

    env = jinja2.Environment(undefined=NullUndefined)

    # stub out cb3 jinja2 functions - they are not important for linting
    #    if we don't stub them out, the ruamel.yaml load fails to interpret them
    #    we can't just use conda-build's api.render functionality, because it would apply selectors
    env.globals.update(dict(compiler=lambda x: x + '_compiler_stub',
                            pin_subpackage=lambda *args, **kwargs: 'subpackage_stub',
                            pin_compatible=lambda *args, **kwargs: 'compatible_pin_stub',
                            cdt=lambda *args, **kwargs: 'cdt_stub',
                            load_file_regex=lambda *args, **kwargs: \
                                    defaultdict(lambda : ''),
                            datetime=datetime,
                            time=time,
                            ))

    with io.open(recipe_meta, 'rt') as fh:
        content = env.from_string(''.join(fh)).render(os=os)
        meta = ruamel.yaml.load(content, ruamel.yaml.RoundTripLoader)
    results = lintify(meta, recipe_dir, conda_forge)
    return results



from __future__ import absolute_import, print_function

import os
from random import choice

from git import Repo

from github import Github
from github.GithubException import GithubException
from github.Organization import Organization
from github.Team import Team

import conda_build.api


def gh_token():
    try:
        with open(os.path.expanduser('~/.conda-smithy/github.token'), 'r') as fh:
            token = fh.read().strip()
    except IOError:
        msg = ('No github token. Go to https://github.com/settings/tokens/new and generate\n'
               'a token with repo access. Put it in ~/.conda-smithy/github.token')
        raise RuntimeError(msg)
    return token


def create_team(org, name, description, repo_names=[]):
    # PyGithub creates secret teams, and has no way of turning that off! :(
    post_parameters = {
        "name": name,
        "description": description,
        "privacy": "closed",
        "permission": "push",
        "repo_names": repo_names
    }
    headers, data = org._requester.requestJsonAndCheck(
        "POST",
        org.url + "/teams",
        input=post_parameters
    )
    return Team(org._requester, headers, data, completed=True)


def add_membership(team, member):
    headers, data = team._requester.requestJsonAndCheck(
        "PUT",
        team.url + "/memberships/" + member
    )
    return (headers, data)


def remove_membership(team, member):
    headers, data = team._requester.requestJsonAndCheck(
        "DELETE",
        team.url + "/memberships/" + member
    )
    return (headers, data)


def has_in_members(team, member):
    status, headers, data = team._requester.requestJson(
        "GET",
        team.url + "/members/" + member
    )
    return status == 204


def get_cached_team(org, team_name, description=""):
    cached_file = os.path.expanduser('~/.conda-smithy/{}-{}-team'.format(org.login, team_name))
    try:
        with open(cached_file, 'r') as fh:
            team_id = int(fh.read().strip())
            return org.get_team(team_id)
    except IOError:
        pass

    team = next((team for team in org.get_teams() if team.name == team_name), None)
    if not team:
        team = create_team(org, team_name, description, [])

    with open(cached_file, 'w') as fh:
        fh.write(str(team.id))

    return team


def create_github_repo(args):
    token = gh_token()
    meta = conda_build.api.render(args.feedstock_directory,
                                  permit_undefined_jinja=True, finalize=False,
                                  bypass_env_check=True, trim_skip=False)[0][0]

    if "parent_recipe" in meta.meta["extra"]:
        feedstock_name = meta.meta["extra"]["parent_recipe"]["name"]
    else:
        feedstock_name = meta.name()

    gh = Github(token)
    user_or_org = None
    if args.user is not None:
        pass
        # User has been defined, and organization has not.
        user_or_org = gh.get_user()
    else:
        # Use the organization provided.
        user_or_org = gh.get_organization(args.organization)

    repo_name = '{}-feedstock'.format(feedstock_name)
    try:
        gh_repo = user_or_org.create_repo(
            repo_name, has_wiki=False,
            description='A conda-smithy repository for {}.'.format(feedstock_name))
        print('Created {} on github'.format(gh_repo.full_name))
    except GithubException as gh_except:
        if gh_except.data.get('errors', [{}])[0].get('message', '') != u'name already exists on this account':
            raise
        gh_repo = user_or_org.get_repo(repo_name)
        print('Github repository already exists.')

    # Now add this new repo as a remote on the local clone.
    repo = Repo(args.feedstock_directory)
    remote_name = args.remote_name.strip()
    if remote_name:
        if remote_name in [remote.name for remote in repo.remotes]:
            existing_remote = repo.remotes[remote_name]
            if existing_remote.url != gh_repo.ssh_url:
                print("Remote {} already exists, and doesn't point to {} "
                      "(it points to {}).".format(remote_name, gh_repo.ssh_url,
                                                  existing_remote.url))
        else:
            repo.create_remote(remote_name, gh_repo.ssh_url)

    if args.add_teams:
        if isinstance(user_or_org, Organization):
            configure_github_team(meta, gh_repo, user_or_org, feedstock_name)


def configure_github_team(meta, gh_repo, org, feedstock_name):

    # Add a team for this repo and add the maintainers to it.
    superlative = [
        'awesome', 'slick', 'formidable', 'awe-inspiring',
        'breathtaking', 'magnificent', 'wonderous', 'stunning',
        'astonishing', 'superb', 'splendid', 'impressive',
        'unbeatable', 'excellent', 'top', 'outstanding', 'exalted',
        'standout', 'smashing'
    ]

    maintainers = set(
        meta.meta.get('extra', {}).get('recipe-maintainers', [])
    )
    maintainers = set(maintainer.lower() for maintainer in maintainers)
    team_name = feedstock_name
    # Try to get team or create it if it doesn't exist.
    team = next((team for team in gh_repo.get_teams() if team.name == team_name), None)
    current_maintainers = []
    if not team:
        team = create_team(
            org,
            team_name,
            'The {} {} contributors!'.format(
                choice(superlative), team_name
            )
        )
        team.add_to_repos(gh_repo)
    else:
        current_maintainers = team.get_members()

    # Add only the new maintainers to the team.
    current_maintainers_handles = set([
        e.login.lower() for e in current_maintainers
    ])
    for new_maintainer in maintainers - current_maintainers_handles:
        add_membership(team, new_maintainer)

    # Mention any maintainers that need to be removed (unlikely here).
    for old_maintainer in current_maintainers_handles - maintainers:
        print(
            "AN OLD MEMBER ({}) NEEDS TO BE REMOVED FROM {}".format(
                old_maintainer, gh_repo
            )
        )

    # Get the all-members team
    team_name = 'all-members'
    description = "All of the awesome {} contributors!".format(org.name)
    all_members_team = get_cached_team(org, team_name, description)
    new_conda_forge_members = set()

    # Add new members to all-members
    for new_member in maintainers - current_maintainers_handles:
        if not has_in_members(all_members_team, new_member):
            print(
                "Adding a new member ({}) to {}. Welcome! :)".format(
                    new_member, org.name
                )
            )
            add_membership(all_members_team, new_member)
            new_conda_forge_members.add(new_member)

    return maintainers, current_maintainers_handles, new_conda_forge_members



#!/usr/bin/env python
from __future__ import print_function
import os
import requests
import time
import sys

import ruamel.yaml

from . import github


# https://circleci.com/docs/api#add-environment-variable

# curl -X POST --header "Content-Type: application/json" -d '{"name":"foo", "value":"bar"}'
# https://circleci.com/api/v1/project/:username/:project/envvar?circle-token=:token

try:
    # Create a token at https://circleci.com/account/api. Put it in circle.token
    with open(os.path.expanduser('~/.conda-smithy/circle.token'), 'r') as fh:
        circle_token = fh.read().strip()
except IOError:
    print('No circle token.  Create a token at https://circleci.com/account/api and\n'
          'put it in ~/.conda-smithy/circle.token')

try:
    with open(os.path.expanduser('~/.conda-smithy/appveyor.token'), 'r') as fh:
        appveyor_token = fh.read().strip()
except IOError:
    print('No appveyor token. Create a token at https://ci.appveyor.com/api-token and\n'
          'Put one in ~/.conda-smithy/appveyor.token')

try:
    anaconda_token = os.environ['BINSTAR_TOKEN']
except KeyError:
    try:
        with open(os.path.expanduser('~/.conda-smithy/anaconda.token'), 'r') as fh:
            anaconda_token = fh.read().strip()
    except IOError:
        print('No anaconda token. Create a token via\n'
              '  anaconda auth --create --name conda-smithy --scopes "repos conda api"\n'
              'and put it in ~/.conda-smithy/anaconda.token')

travis_endpoint = 'https://api.travis-ci.org'

def travis_headers():
    headers = {
       # If the user-agent isn't defined correctly, we will recieve a 403.
       'User-Agent': 'Travis/1.0',
       'Accept': 'application/json',
       'Content-Type': 'application/json',
       'Travis-API-Version': '3'
    }
    travis_token = os.path.expanduser('~/.conda-smithy/travis.token')
    if not os.path.exists(travis_token):
        # We generally want the V3 API, but can currently only auth with V2:
        # https://github.com/travis-ci/travis-ci/issues/9273#issuecomment-370474214
        v2_headers = headers.copy()
        v2_headers['Accept'] = 'application/vnd.travis-ci.2+json'
        del v2_headers['Travis-API-Version']

        url = '{}/auth/github'.format(travis_endpoint)
        data = {"github_token": github.gh_token()}
        response = requests.post(url, json=data, headers=v2_headers)
        if response.status_code != 201:
            response.raise_for_status()
        token = response.json()['access_token']
        with open(travis_token, 'w') as fh:
            fh.write(token)
        # TODO: Set the permissions on the file.
    else:
        with open(travis_token, 'r') as fh:
            token = fh.read().strip()

    headers['Authorization'] = 'token {}'.format(token)
    return headers


def add_token_to_circle(user, project):
    url_template = ('https://circleci.com/api/v1.1/project/github/{user}/{project}/envvar?'
                    'circle-token={token}')
    url = url_template.format(token=circle_token, user=user, project=project)
    data = {'name': 'BINSTAR_TOKEN', 'value': anaconda_token}
    response = requests.post(url, data)
    if response.status_code != 201:
        raise ValueError(response)


def add_project_to_circle(user, project):
    headers = {'Content-Type': 'application/json',
               'Accept': 'application/json'}
    url_template = ('https://circleci.com/api/v1.1/project/github/{component}?'
                    'circle-token={token}')

    # Note, we used to check to see whether the project was already registered, but it started
    # timing out once we had too many repos, so now the approach is simply "add it always".

    url = url_template.format(component='{}/{}/follow'.format(user, project).lower(), token=circle_token)
    response = requests.post(url, headers={})
    # It is a strange response code, but is doing what was asked...
    if response.status_code != 400:
        response.raise_for_status()

    # Note, here we are using a non-public part of the API and may change
    # Enable building PRs from forks
    url = url_template.format(component='{}/{}/settings'.format(user, project).lower(), token=circle_token)
    # Disable CircleCI secrets in builds of forked PRs explicitly.
    response = requests.put(url, headers=headers, json={'feature_flags':{'forks-receive-secret-env-vars':False}})
    if response.status_code != 200:
        response.raise_for_status()
    # Enable CircleCI builds on forked PRs.
    response = requests.put(url, headers=headers, json={'feature_flags':{'build-fork-prs':True}})
    if response.status_code != 200:
        response.raise_for_status()

    print(' * {}/{} enabled on CircleCI'.format(user, project))


def add_project_to_appveyor(user, project):
    headers = {'Authorization': 'Bearer {}'.format(appveyor_token),
               }
    url = 'https://ci.appveyor.com/api/projects'

    response = requests.get(url, headers=headers)
    if response.status_code != 201:
        response.raise_for_status()
    repos = [repo['repositoryName'].lower() for repo in response.json()]

    if '{}/{}'.format(user, project).lower() in repos:
        print(' * {}/{} already enabled on appveyor'.format(user, project))
    else:
        data = {'repositoryProvider': 'gitHub', 'repositoryName': '{}/{}'.format(user, project)}
        response = requests.post(url, headers=headers, data=data)
        if response.status_code != 201:
            response.raise_for_status()
        print(' * {}/{} has been enabled on appveyor'.format(user, project))


def appveyor_encrypt_binstar_token(feedstock_directory, user, project):
    headers = {'Authorization': 'Bearer {}'.format(appveyor_token)}
    url = 'https://ci.appveyor.com/api/account/encrypt'
    response = requests.post(url, headers=headers, data={"plainValue": anaconda_token})
    if response.status_code != 200:
        raise ValueError(response)

    forge_yaml = os.path.join(feedstock_directory, 'conda-forge.yml')
    if os.path.exists(forge_yaml):
        with open(forge_yaml, 'r') as fh:
            code = ruamel.yaml.load(fh, ruamel.yaml.RoundTripLoader)
    else:
        code = {}

    # Code could come in as an empty list.
    if not code:
        code = {}

    code.setdefault('appveyor', {}).setdefault('secure', {})['BINSTAR_TOKEN'] = response.content.decode('utf-8')
    with open(forge_yaml, 'w') as fh:
        fh.write(ruamel.yaml.dump(code, Dumper=ruamel.yaml.RoundTripDumper))


def appveyor_configure(user, project):
    """Configure appveyor so that it skips building if there is no appveyor.yml present."""
    headers = {'Authorization': 'Bearer {}'.format(appveyor_token)}
    # I have reasons to believe this is all AppVeyor is doing to the API URL.
    project = project.replace('_', '-').replace('.', '-')
    url = 'https://ci.appveyor.com/api/projects/{}/{}/settings'.format(user, project)
    response = requests.get(url, headers=headers)
    if response.status_code != 200:
        raise ValueError(response)
    content = response.json()
    settings = content['settings']
    for required_setting in (
        u'skipBranchesWithoutAppveyorYml',
        u'rollingBuildsOnlyForPullRequests',
        u'rollingBuilds',
    ):
        if not settings[required_setting]:
            print('{: <30}: Current setting for {} = {}.'
                  ''.format(project, required_setting, settings[required_setting]))
        settings[required_setting] = True

    url = 'https://ci.appveyor.com/api/projects'.format(user, project)
    response = requests.put(url, headers=headers, json=settings)
    if response.status_code != 204:
        raise ValueError(response)


def travis_wait_until_synced(ignore=False):
    headers = travis_headers()
    is_sync_url = '{}/user'.format(travis_endpoint)
    for c in range(20):
        response = requests.get(is_sync_url, headers=headers)
        content = response.json()
        print(".", end="")
        sys.stdout.flush()
        if ("is_syncing" in content and content["is_syncing"] == False):
            break
        time.sleep(6)
    else:
        if ignore:
            print(" * Travis is being synced by somebody else. Ignoring")
        else:
            raise RuntimeError("Syncing has not finished for two minutes now.")
    print("")
    return content


def travis_get_repo_info(user, project, show_error=False):
    headers = travis_headers()
    url = '{}/repo/{user}%2F{project}'.format(travis_endpoint, user=user, project=project)
    response = requests.get(url, headers=headers)
    try:
        response.raise_for_status()
        content = response.json()
        return content
    except requests.HTTPError as e:
        if show_error:
            print(e)
    return {}


def add_project_to_travis(user, project):
    headers = travis_headers()

    repo_info = travis_get_repo_info(user, project, show_error=False)
    if not repo_info:
        # Travis needs syncing. Wait until other syncs are finished.
        print(" * Travis: checking if there's a syncing already", end="")
        sys.stdout.flush()
        user_info = travis_wait_until_synced(ignore=True)
        repo_info = travis_get_repo_info(user, project, show_error=False)
        if not repo_info:
            print(" * Travis doesn't know about the repo, syncing (takes a few seconds).", end="")
            sys.stdout.flush()
            sync_url = '{}/user/{}/sync'.format(travis_endpoint, user_info['id'])
            response = requests.post(sync_url, headers=headers)
            if response.status_code != 409:
                # 409 status code is for indicating that another synching might be happening at the
                # same time. This can happen in conda-forge/staged-recipes when two master builds
                # start at the same time
                response.raise_for_status()
            travis_wait_until_synced(ignore=False)
            repo_info = travis_get_repo_info(user, project)

    if not repo_info:
        msg = ('Unable to register the repo on Travis\n'
               '(Is it down? Is the "{}/{}" name spelt correctly? [note: case sensitive])')
        raise RuntimeError(msg.format(user, project))

    if repo_info['active'] is True:
        print(' * {}/{} already enabled on travis-ci'.format(user, project))
    else:
        repo_id = repo_info['id']
        url = '{}/repo/{}/activate'.format(travis_endpoint, repo_id)
        response = requests.post(url, headers=headers)
        response.raise_for_status()
        print(' * {}/{} registered on travis-ci'.format(user, project))


def travis_token_update_conda_forge_config(feedstock_directory, user, project):
    item = 'BINSTAR_TOKEN="{}"'.format(anaconda_token)
    slug = "{}%2F{}".format(user, project)

    forge_yaml = os.path.join(feedstock_directory, 'conda-forge.yml')
    if os.path.exists(forge_yaml):
        with open(forge_yaml, 'r') as fh:
            code = ruamel.yaml.load(fh, ruamel.yaml.RoundTripLoader)
    else:
        code = {}

    # Code could come in as an empty list.
    if not code:
        code = {}

    code.setdefault('travis', {}).setdefault('secure', {})['BINSTAR_TOKEN'] = (
        travis_encrypt_binstar_token(slug, item)
    )
    with open(forge_yaml, 'w') as fh:
        fh.write(ruamel.yaml.dump(code, Dumper=ruamel.yaml.RoundTripDumper))


def travis_encrypt_binstar_token(repo, string_to_encrypt):
    # Copyright 2014 Matt Martz <matt@sivel.net>
    # All Rights Reserved.
    #
    #    Licensed under the Apache License, Version 2.0 (the "License"); you may
    #    not use this file except in compliance with the License. You may obtain
    #    a copy of the License at
    #
    #         http://www.apache.org/licenses/LICENSE-2.0
    #
    #    Unless required by applicable law or agreed to in writing, software
    #    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
    #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
    #    License for the specific language governing permissions and limitations
    #    under the License.
    from Crypto.PublicKey import RSA
    from Crypto.Cipher import PKCS1_v1_5
    import base64

    keyurl = 'https://api.travis-ci.org/repo/{0}/key_pair/generated'.format(repo)
    r = requests.get(keyurl, headers=travis_headers())
    r.raise_for_status()
    public_key = r.json()['public_key']
    key = RSA.importKey(public_key)
    cipher = PKCS1_v1_5.new(key)
    return base64.b64encode(cipher.encrypt(string_to_encrypt.encode())).decode('utf-8')


def travis_configure(user, project):
    """Configure travis so that it skips building if there is no .travis.yml present."""
    headers = travis_headers()

    repo_info = travis_get_repo_info(user, project)
    repo_id = repo_info['id']

    if repo_info['active'] is not True:
        raise ValueError(
            "Repo {user}/{project} is not active on Travis CI".format(user=user, project=project)
        )

    settings = [
        ("builds_only_with_travis_yml", True),
        ("auto_cancel_pull_requests", True),
    ]
    for name, value in settings:
        url = '{}/repo/{repo_id}/setting/{name}'.format(travis_endpoint, repo_id=repo_id, name=name)
        data = {"setting.value": value}
        response = requests.patch(url, json=data, headers=headers)
        if response.status_code != 204:
            response.raise_for_status()


def get_conda_hook_info(hook_url, events):
    payload = {
          "name": "web",
          "active": True,
          "events": events,
          "config": {
            "url": hook_url,
            "content_type": "json"
          }
        }

    return hook_url, payload


def add_conda_forge_webservice_hooks(user, repo):
    if user != 'conda-forge':
        print('Unable to register {}/{} for conda-linting at this time as only '
              'conda-forge repos are supported.'.format(user, repo))

    headers = {'Authorization': 'token {}'.format(github.gh_token())}
    url = 'https://api.github.com/repos/{}/{}/hooks'.format(user, repo)

    # Get the current hooks to determine if anything needs doing.
    response = requests.get(url, headers=headers)
    response.raise_for_status()
    registered = response.json()
    hook_by_url = {hook['config'].get('url'): hook for hook in registered
                   if 'url' in hook['config']}

    hooks = [
        get_conda_hook_info(
            "http://conda-forge.herokuapp.com/conda-linting/hook",
            [
                "pull_request"
            ]
        ),
        get_conda_hook_info(
            "http://conda-forge.herokuapp.com/conda-forge-feedstocks/hook",
            [
                "push", "repository"
            ]
        ),
        get_conda_hook_info(
            "http://conda-forge.herokuapp.com/conda-forge-teams/hook",
            [
                "push", "repository"
            ]
        ),
        get_conda_hook_info(
            "http://conda-forge.herokuapp.com/conda-forge-command/hook",
            [
                "pull_request_review", "pull_request",
                "pull_request_review_comment", "issue_comment", "issues",
            ]
        ),
    ]

    for hook in hooks:
        hook_url, payload = hook
        if hook_url not in hook_by_url:
            response = requests.post(url, json=payload, headers=headers)
            if response.status_code != 200:
                response.raise_for_status()


if __name__ == '__main__':
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument("user")
    parser.add_argument("project")
    args = parser.parse_args()

#    add_project_to_circle(args.user, args.project)
#    add_project_to_appveyor(args.user, args.project)
#    add_project_to_travis(args.user, args.project)
#    appveyor_encrypt_binstar_token('../udunits-delme-feedstock', args.user, args.project)
#    appveyor_configure('conda-forge', 'glpk-feedstock')
#    travis_token_update_conda_forge_config('../udunits-delme-feedstock', args.user, args.project)
    add_conda_forge_webservice_hooks(args.user, args.project)
    print('Done')
